#include <qdom.h>
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qimage.h>
#include <qstring.h>
#include <qptrlist.h>

#ifdef HAVE_CONFIG
#include "config.h"
#else
#include "defaults.h"
#endif

#include "book.h"
#include "main.h"
#include "picture.h"



Book::Book(const QString& file) 
{
    pictures_ = new QPtrList<Picture>;

    /* Is it an existing file */
    if (QFileInfo(file).exists()) {
        readFromFile(file);
        initAttributes();
    }
    else {
        QDomDocument doc("index_info");
        info_root_ = doc.createElement("index");
        info_html_ = doc.createElement("settings");
        setFile(file); 
        initAttributes();
    }
}


Book::~Book()
{
    delete pictures_;
}


void Book::setFile(const QString& file) 
{
    file_ = QFileInfo(file).fileName();
    dir_ = QFileInfo(file).dirPath();
}


void Book::initAttributes()
{
    setAttribute("version",SPIT_VERSION,true);
    setAttribute("title","Untitled",false);
    setAttribute("date","",false);
        
    /* Thumbs settings */
    setNumAttribute("thumbs_width",200,false);
    setNumAttribute("thumbs_height", 150,false);
    setNumAttribute("thumbs_quality", 75,false);
    setNumAttribute("thumbs_scalemode",(int) QImage::ScaleMin,false);
    setAttribute("thumbs_dir","thumbs",false);
    html_setNumAttribute("thumbs_per_line", 3,false);
    html_setNumAttribute("lines_per_page", 3,false);
        
    /* Slideshow settings */
    setNumAttribute("slides_width",800,false);
    setNumAttribute("slides_height", 600,false);
    setNumAttribute("slides_quality", 75,false);
    setNumAttribute("slides_scalemode",(int) QImage::ScaleMin,false);
    setAttribute("slides_dir","slides",false);
    setBoolAttribute("slides_create",true,false);
                                     
    /* Originals settings */
    setNumAttribute("originals_width",1280,false);
    setNumAttribute("originals_height", 1024,false);
    setNumAttribute("originals_quality", 95,false);
    setNumAttribute("originals_scalemode",(int) QImage::ScaleMin,false);
    setBoolAttribute("originals_scale",false,false);
    setBoolAttribute("originals_create",true,false);
    setAttribute("originals_dir","originals",false);
      
    /* HTML settings */
    setAttribute("html_dir","",false);
    html_setAttribute("linkup","",false);

    html_setAttribute("template", LIBDIR "templates/speek.xsl",false); 
    html_setAttribute("stylesheet_file", LIBDIR "templates/speek.css",false); 
    html_setAttribute("stylesheet", "speek.css",false); 

    /* Misc Settings */
    setBoolAttribute("desc_default",false,false);
#ifdef Q_OS_MACX
    setAttribute("editor","open",false);
    setAttribute("browser","open",false);
#else
    setAttribute("editor","gimp-remote -n",false);
    setAttribute("browser","mozilla",false);
#endif
}
    

void Book::readFromFile(const QString& file) 
{
    if (file.isEmpty() ) return;
    
    /* Initialize the XML document */
    QDomDocument doc = QDomDocument("index_info");
    QFile f(file );
    if (!f.open(IO_ReadOnly)) {
        qFatal("Error in opening descriptions file.");
    }
    int err_line,err_column; 
    QString err_msg;
    if ( !doc.setContent( &f, &err_msg, &err_line, &err_column) ) {
        f.close();
        qFatal("Parse error in " + file + " (l." + QString::number(err_line) 
                                   + ",c." + QString::number(err_column)
                                   + "): " + err_msg);
    }
    f.close();

    /* Initialize pics */
    QDomNodeList list = doc.elementsByTagName("picture");
    for (unsigned int i=0 ; i < list.count(); i++) {
        QDomNode picNode = list.item(i);
        new Picture(this, picNode.toElement().attribute("name"),
                          picNode.toElement().attribute("description"));
    }

    /* Initialize root */
    info_root_ = doc.documentElement();

    /* Initialize HTML generator settings */
    info_html_ = doc.elementsByTagName("settings").item(0).toElement();

    setFile(file);
}


void Book::writeToFile() const
{
    writeToFile(QDir(dir_).filePath(file_));
}


void Book::writeToFile(const QString& target) const
{
    QDomNamedNodeMap attrs;

    QDomDocument doc = QDomDocument("index_info");

    /* Create a root node */
    QDomElement rootNode = doc.createElement("index");
    attrs = info_root_.attributes();
    for (unsigned int i=0 ; i < attrs.count(); i++) 
        rootNode.setAttributeNode(attrs.item(i).toAttr());      
    doc.appendChild(rootNode);
  
    /* Generator element */
    QDomElement genNode = doc.createElement("generator");
    genNode.setAttribute("name","xslt");
    rootNode.appendChild(genNode);

    /* Generator settings element */
    QDomElement genSettingsNode = doc.createElement("settings");
    attrs = info_html_.attributes();
    for (unsigned int j=0 ; j < attrs.count(); j++) 
        genSettingsNode.setAttributeNode(attrs.item(j).toAttr());      
    genNode.appendChild(genSettingsNode);

    /* Filenames & descriptions */
    QPtrListIterator<Picture> it(*pictures());
    for ( ; it.current(); ++it ) {
        QDomElement pic = doc.createElement("picture");
        pic.setAttribute("name",(*it)->fileName());
        pic.setAttribute("description",(*it)->description());
        rootNode.appendChild(pic);
    }
        
    /* Write to file */
    QString output = doc.toString();
    QFile outFile(target);
    if (outFile.open(IO_WriteOnly|IO_Translate)) {
        outFile.writeBlock(output,output.length());
        outFile.close();
    }
    else
        qWarning("Can't open index file " + target);
}


void Book::addPicture(Picture* picture)
{
    if (!containsPicture(picture->fileName())) 
        pictures()->append(picture);
    else
        qDebug(QString("Book allready contains%1" ).arg(picture->fileName()));
}


void Book::removePicture(Picture* picture)
{
    pictures()->remove(pictures()->find(picture));
}


void Book::movePictureForward(Picture* picture)
{
    int i = pictures()->find(picture);
    Q_ASSERT(i >= 0);
    if ((unsigned int) i < pictures()->count()-1) {
        pictures()->remove(i);
        pictures()->insert(i+1,picture);
    }
}

void Book::movePictureBackward(Picture* picture)
{
    int i = pictures()->find(picture);
    Q_ASSERT(i >= 0);
    if (i > 0) {
        pictures()->remove(i);
        pictures()->insert(i-1,picture);
    }
}


int Book::getPictureIndex(const Picture* picture) const 
{
    return pictures()->find(picture);
}


bool Book::containsPicture(const QString& picture) const
{
    QPtrListIterator<Picture> it(*pictures());
    for ( ; it.current(); ++it )
        if ((*it)->fileName() == picture) return true;

    return false;
}
