#include <qapplication.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qthread.h>

#ifdef HAVE_IMAGEMAGICKPP
#include <Magick++.h>
#include <string.h>
using namespace Magick;
#endif

#include "imagemanager.h"
#include "picture.h"


/********************* Threads used in the image manager *********************/

class ImageScaleThread : public QThread
{
  public:
    ImageScaleThread (const QString& s, const QString& t,
                      int w, int h, int q, QImage::ScaleMode m) :
        s_(s), t_(t), w_(w), h_(h), q_(q), m_(m)  { }

    virtual void run() { ImageManager::scale(s_,t_,w_,h_,q_,m_); }

  private:
    QString s_,t_;
    int w_,h_,q_;
    QImage::ScaleMode m_;
};


class ImageRotateThread : public QThread
{
  public:
    ImageRotateThread (const QString& s, double deg) : s_(s), deg_(deg) { }
    virtual void run() { ImageManager::rotate(s_,deg_); }

  private:
    QString s_;
    double deg_;
};





/****************************************************************************/


QPixmap ImageManager::getThumb(const Picture* picture)
{
    /* Refresh the thumbnail if necessary */
    refresh(picture);
#ifdef HAVE_IMAGEMAGICKPP
    Image image;
    image.read(picture->thumbPath().latin1());
    uchar* pixels = new uchar[image.columns()*image.rows()*4];
    image.write(0,0,image.columns(),image.rows(),"BGRA",CharPixel,pixels); 
    QImage img((uchar*) pixels,image.columns(),image.rows(),32,0,3,
               QImage::IgnoreEndian);
#else
    QImage img(picture->thumbPath());
#endif

    return QPixmap(img);
}


void ImageManager::importPicture(const Picture* picture, const QString& file)
{
    if (QFileInfo(picture->filePath()).dir().exists()
        || (QFileInfo(picture->filePath()).dir().mkdir("")))
        copyImage(file,picture->filePath());
    else
        qWarning("Error creating originals dir");
}


void ImageManager::rotate(const QString& picture, double degrees)
{
#ifdef HAVE_IMAGEMAGICKPP
    Image image;
    image.read(picture.latin1());
    /* FIXME: does the quality needs to be set ?  */
    image.rotate(degrees);
    image.write(picture.latin1());
#endif
}


void ImageManager::refresh(const Picture* picture)
{
    /* Check if the thumb is ok */
    if (!QFileInfo(picture->thumbPath()).exists()
        || isDirty(picture->filePath(),picture->thumbPath()))
        rebuildThumb(picture);

    /* Check if the slide is ok */
    QFileInfo sfi = QFileInfo(picture->slidePath());
    if (picture->hasSlide() && ( !sfi.exists() 
            || isDirty(picture->filePath(),picture->slidePath()))) 
        rebuildSlide(picture);

    /* Check if slide has to be removed */
    if (!picture->hasSlide() && sfi.exists()) {
        if (sfi.dir().remove(sfi.fileName())) { 
            if (sfi.dir().exists() && sfi.dir().count() == 2 
                                   && !sfi.dir().rmdir("")) 
                qWarning("Error deleting empty slide dir");
        }
        else 
            qWarning("Error deleting slide file");
    }
}


void ImageManager::rebuildThumb(const Picture* picture)
{
    if (QFileInfo(picture->thumbPath()).dir().exists() 
        || (QFileInfo(picture->thumbPath()).dir().mkdir("")))
        scaleImage(picture->filePath(),picture->thumbPath(),
                   picture->thumbWidth(), picture->thumbHeight(),
                   picture->thumbQuality(), picture->thumbScaleMode());
    else qWarning("Error creating thumb dir");
}


void ImageManager::rebuildSlide(const Picture* picture)
{
    if (picture->hasSlide())
        if ( QFileInfo(picture->slidePath()).dir().exists() 
             || (QFileInfo(picture->slidePath()).dir().mkdir(""))) {
            scaleImage(picture->filePath(),picture->slidePath(),
                       picture->slideWidth(), picture->slideHeight(),
                       picture->slideQuality(), picture->slideScaleMode());
        }
        else qWarning("Error creating slide dir");
}


bool ImageManager::isReadable(const QString& file)
{
    if (!QFileInfo(file).exists()) 
        return false;

    const char* image_format = QImage::imageFormat(file);
    if (!image_format) {
         qWarning(
            "Unrecognized image format. If you have the plugin\n"
            "for this format, be sure to have the QTDIR environment\n"
            "variable set correctly.");
        return false;
    }
    
    if (!QImageIO::inputFormats().contains(QString(image_format)) || 
        !QImageIO::outputFormats().contains(QString(image_format))) {
        QMessageBox::warning(0, "Spit",
            QString(QObject::tr("No %1 support in Qt library.\n"
                "Be sure to have the QTDIR environment variable\n"
                "set correctly.")).arg(QString(image_format)),
            QMessageBox::Ok | QMessageBox::Default,
            QMessageBox::NoButton);
        return false;
    }

    return true;
}


bool ImageManager::isDirty(const QString& oldFile, const QString& newFile) 
{
    QFileInfo source(oldFile);
    QFileInfo target(newFile);

    return (!target.exists() || source.lastModified() > target.lastModified());
}


void ImageManager::scaleImage(const QString& source, const QString& target,
                              int w, int h, int quality, QImage::ScaleMode mode)
{
    ImageScaleThread t (source,target,w,h,quality,mode);
    t.start();
    while (t.running()) {
        qApp->processEvents();
        t.wait(100);
    }
}


void ImageManager::rotatePicture(const Picture* picture, double deg) 
{
    ImageRotateThread t(picture->filePath(),deg);
    t.start();
    while (t.running()) {
        qApp->processEvents();
        t.wait(100);
    }
}


void ImageManager::scale(const QString& sourceFile, const QString& targetFile,
                              int w, int h, int quality, QImage::ScaleMode mode)
{
    qDebug(QString("Scaling %1 to %2").arg(sourceFile).arg(targetFile));
    if (!isReadable(sourceFile)) 
        return;

#ifdef HAVE_IMAGEMAGICKPP
    Image image;
    image.read(sourceFile.latin1());
    /* FIXME: use scalemode to determine size. Sample or scale ? */
    image.scale(QString("%1x%2").arg(w).arg(h).latin1());
    image.quality(quality);
    image.write(targetFile.latin1());
#else
    /* Load source image */
    const char* image_format = QImage::imageFormat(sourceFile);
    QImage image(sourceFile);
    Q_ASSERT(!image.isNull());
                
    /* Check if images really needs to be rescaled */
    if ((mode == QImage::ScaleMin && 
            (image.width() > w || image.height() > h))
        || (mode == QImage::ScaleMax &&
            (image.width() < w || image.height() < h))
        || (mode == QImage::ScaleFree &&
            (image.width() != w || image.height() != h))) {
        /* Rescale image & save */
        QImage new_image = image.smoothScale(w,h,mode);
        new_image.save(targetFile,image_format,quality);
    }
    else { 
        qDebug("Image doesn't need to be scaled => Copying image");
        copyImage(sourceFile, targetFile);     
    }
#endif
}


void ImageManager::copyImage(const QString& sourceFile, 
                             const QString& targetFile)
{
    /* Check if the file has to be copied */
    if (QDir(QFileInfo(sourceFile).dirPath()).canonicalPath() !=
        QDir(QFileInfo(targetFile).dirPath()).canonicalPath()) {
        /* Open the source & target files */
        QFile source_file(sourceFile);
        QFile target_file(targetFile);
        qDebug(QString("Copying %1 to %2").arg(sourceFile).arg(targetFile));
        if (source_file.open(IO_ReadOnly) 
            && target_file.open(IO_WriteOnly)) {
            /* Copy all the data */
            char data[4096];
            while (!source_file.atEnd()) {
                Q_LONG read = source_file.readBlock(data,4096);
                target_file.writeBlock(data,read);
            }
        }
        else 
            qWarning(QString("Error copying %1 to %2").arg(sourceFile).
                                                       arg(targetFile));
    }
    else 
        qDebug("Source & destination are the same => not copying");
}


void ImageManager::cleanup(const Picture* picture, bool cleanupFile)
{
    qWarning("Cleaning up thumbs & slide images");

    /* Remove thumb file */
    QFileInfo tfi = QFileInfo(picture->thumbPath());
    if (tfi.exists() && !tfi.dir().remove(tfi.fileName()))
        qWarning("Error deleting thumb file");
    // FIXME: count == 2 is not really clean !
    if (tfi.dir().exists() && tfi.dir().count() == 2)
        tfi.dir().rmdir("");

    /* Remove slide file */
    QFileInfo sfi = QFileInfo(picture->slidePath());
    if (sfi.exists() && !sfi.dir().remove(sfi.fileName()))
        qWarning("Error deleting slide file");
    if (sfi.dir().exists() && sfi.dir().count() == 2)
        sfi.dir().rmdir("");


    /* Remove original file */
    QFileInfo ofi = QFileInfo(picture->filePath());
    if (cleanupFile && ofi.exists() && !ofi.dir().remove(ofi.fileName()))
         qWarning("Error deleting original file");
    if (ofi.dir().exists() && ofi.dir().count() == 2)
        ofi.dir().rmdir("");


}


/* All image formats supported */
QString ImageManager::supportedImageFormats = 
    "*.jpg *.JPG *.png *.PNG *.gif *.GIF";
