#include <qapplication.h>
#include <qaction.h>
#include <qfiledialog.h>
#include <qglobal.h>
#include <qiconview.h>
#include <qinputdialog.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qprogressdialog.h>
#include <qstringlist.h>
#include <qstatusbar.h>
#include <qptrlist.h>

#include "book.h"
#include "browser.h"
#include "htmlgenerator.h"
#include "indexwindow.h"
#include "imagemanager.h"
#include "main.h"
#include "picture.h"
#include "picturecontextmenu.h"
#include "pictureitem.h"
#include "settingsdialog.h"


IndexWindow::IndexWindow(QString bookFile) : IndexWindowBase(), book_(0)
{
    if (!bookFile.isNull()) {
        if (QFileInfo(bookFile).exists()) {
            /* Initialize index */
            book_ = new Book(bookFile);

            /* Enable all widgets */
            enableAll();

            /* Set text & title */
            text_title->setText(book()->getAttribute("title"));
            text_date->setText(book()->getAttribute("date"));

            /* Initialize picture list */
            updatePictures();
        }
        else 
           qFatal("Error: " + bookFile + " does not exist.");
    }    

    /* Separate Help menu from the rest */
    menuBar()->insertSeparator(menuBar()->count()-1);
}


QPtrList<PictureItem>* IndexWindow::getSelectedPictures()
{   
    QPtrList<PictureItem>* list = new QPtrList<PictureItem>;
    list->setAutoDelete(FALSE);

    for (QIconViewItem *el = view_pictures->firstItem(); el; 
                        el = el->nextItem()) 
        if (el->isSelected())
            list->append((PictureItem*) el);
    
    return list;
}   

void IndexWindow::action_newIndex_activated()
{
    /* Get target file */
    QString file = QFileDialog::getSaveFileName("","Index (*.ind)",
                                    0,"new","New");
    
    if (file.isEmpty()) return;
    
    /* Check if the filename has an extension. If not, add .bk */
    if (QFileInfo(file).extension().isEmpty()) 
        file += ".ind";
    
    /* Check if the file already exists */
    if (QFileInfo(file).exists()
             && (QMessageBox::information(this, "Spit",
                QString(tr("File '%1' already exists.\n"
                        "Do you want to overwrite the file ?")).arg(file), 
                QMessageBox::Ok, 
                QMessageBox::Cancel | QMessageBox::Default | 
                QMessageBox::Escape) == QMessageBox::Cancel)) {
        return;
    }
    if (QFileInfo(file).exists())
        QFile(file).remove();
    
    /* Close open book */
    action_closeIndex_activated();
    if (book_) return; /* Check if the user really exited */
    
    /* Create new book */
    book_ = new Book(file);
    
    /* Get target file */
    SettingsDialog d;
    d.setBook(book());
    d.exec();

    if (d.result() == QDialog::Accepted) {
        /* Enable everything */
        enableAll();
    }
    else {
        /* Reset info & manager */
        book_ = 0;
    }
}


void IndexWindow::action_openIndex_activated()
{
    QString file = QFileDialog::getOpenFileName( "Index files (*.ind)",
            "",0,"open file dialog", tr("Open Index ..."));

    if (!file.isEmpty()) {
        /* Close current index */
        action_closeIndex_activated();

        /* Create new info objects */
        book_ = new Book(file);

        /* Enable everything */
        enableAll();

        /* Set text values */
        text_title->setText(book()->getAttribute("title"));
        text_date->setText(book()->getAttribute("date"));

        /* Show pictures */
        updatePictures();
    }
}

void IndexWindow::action_saveIndex_activated()
{
    /* Update images */
/*    QProgressDialog progress(tr("Refreshing pictures ..."),0,
                        view_pictures->count()+1,0,"progress",true);
    progress.setMinimumDuration(0);
    int i=0;
    for (QIconViewItem *el=view_pictures->firstItem(); el; el=el->nextItem()) {
        progress.setProgress(i++);
        qApp->processEvents();
        ((PictureItem*) el)->refresh();
    }*/
    
    /* Write book to file */
//    progress.setProgress(i++);
//    progress.setLabelText(tr("Saving file ..."));
//    qApp->processEvents();
    book()->writeToFile();

    /* Cleanup progress */
//    progress.setProgress(view_pictures->count()+1);
}

void IndexWindow::action_saveIndexAs_activated()
{
    QString target = QFileDialog::getSaveFileName("",tr("Index files (*.ind)"),
                        0,"save",tr("Save as ..."));
    if (!target.isNull()) {
        book()->setFile(target);
        action_saveIndex_activated();
    }
}


void IndexWindow::action_rebuildPictures_activated()
{    
    QPtrList<PictureItem> *list = getSelectedPictures();
    
    /* Rebuild images */
    QProgressDialog progress(tr("Rebuilding pictures ..."),0,
        list->count(),0,"progress",true);
    progress.setMinimumDuration(0);
    int i=0;
    for (QPtrListIterator<PictureItem> it(*list); it.current(); ++it ) {
        progress.setProgress(i++);
        qApp->processEvents();
        (*it)->rebuild();
    }
    progress.setProgress(list->count());
    delete list;
    
    view_pictures->sort();
}


void IndexWindow::action_refreshPictures_activated()
{    
    QPtrList<PictureItem> *list = getSelectedPictures();
    
    /* Rebuild images */
    QProgressDialog progress(tr("Refreshing pictures ..."),0,
        list->count(),0,"progress",true);
    progress.setMinimumDuration(0);
    int i=0;
    for (QPtrListIterator<PictureItem> it(*list); it.current(); ++it ) {
        progress.setProgress(i++);
        qApp->processEvents();
        (*it)->refresh();
    }
    progress.setProgress(list->count());
    delete list;
    
    view_pictures->sort();
}


void IndexWindow::action_generateHTML_activated()
{
    /* Update images */
    QProgressDialog progress(tr("Refreshing pictures ..."),0,
            view_pictures->count()+1,0,"progress",true);
    progress.setMinimumDuration(0);
    int i=0;
    for (QIconViewItem *el=view_pictures->firstItem(); el; el=el->nextItem()) {
        progress.setProgress(i++);
        qApp->processEvents();
        ((PictureItem*) el)->refresh();
    }

    /* Update HTML */
    progress.setProgress(i++);
    progress.setLabelText(tr("Generating HTML ..."));
    qApp->processEvents();
    createHTML(book());

    progress.setProgress(view_pictures->count()+1);
}


void IndexWindow::action_previewHTML_activated()
{
    Browser::view(book());
}


void IndexWindow::action_closeIndex_activated()
{
    /* Check if changes have to be saved */
    if (book_) 
        switch(QMessageBox::information(this, "Spit",
                tr("Do you want to save your\n"
                   "changes before closing ?"), 
                QMessageBox::Yes | QMessageBox::Default,
                QMessageBox::No,
                QMessageBox::Cancel | QMessageBox::Escape)) {
            case QMessageBox::Yes : action_saveIndex_activated(); break;
            case QMessageBox::No : break;
            case QMessageBox::Cancel : return;
        }

    /* Clear everything */
    disableAll();

    /* Delete bookinfo */
    if (book_) {
        delete(book_); 
        book_ = 0;
    }
}

void IndexWindow::action_importPictures_activated()
{
    QStringList pics = QFileDialog::getOpenFileNames(
            "Images (" + ImageManager::supportedImageFormats + ")",
            "",0,"open file dialog",
            tr("Import pictures ..."));
    QProgressDialog progress(tr("Importing pictures ..."),tr("Cancel"),
            pics.count(), 0, "progress", true );
    progress.setMinimumDuration(0);
    int i=0;
    for(QStringList::Iterator it = pics.begin(); it != pics.end(); it++) {
#if QT_VERSION < 0x030200
        if (progress.wasCancelled()) 
#else
        if (progress.wasCanceled()) 
#endif
            break;
        progress.setProgress(i++); 
        qApp->processEvents();
        QString file = QFileInfo(*it).fileName();
        Picture* picture;
        if (book()->getBoolAttribute("desc_default")) 
            picture = new Picture(book(),file,QFileInfo(file).baseName());
        else 
            picture = new Picture(book(),file,"");
        ImageManager::importPicture(picture,*it); 
        new PictureItem(view_pictures,picture);
    }
    progress.setProgress(pics.count());

    view_pictures->sort();
    book()->writeToFile();
}


void IndexWindow::text_title_textChanged(const QString& title)
{
    if (book_)
        book()->setAttribute("title",title);
}


void IndexWindow::text_date_textChanged(const QString& date)
{
    if (book_)
        book()->setAttribute("date",date);
}
         


void IndexWindow::view_pictures_itemRenamed(QIconViewItem* it, 
                                            const QString& newDesc)
{
    ((PictureItem*) it)->setDescription(newDesc);
    view_pictures->sort();
}


void IndexWindow::view_pictures_selectionChanged()
{
    /* Check if something is highlighted */
    bool enable = false;
    QIconViewItem *el = view_pictures->firstItem();
    while (!enable && el) {
        PictureItem* p = (PictureItem*) el;
        enable = enable || p->isSelected();
        el = el->nextItem();
    }
    if (action_deletePictures->isEnabled() != enable) {
        /* Change everything */
        action_deletePictures->setEnabled(enable);
        action_rebuildPictures->setEnabled(enable);
        action_refreshPictures->setEnabled(enable);
        action_invertSelection->setEnabled(enable);
        action_moveForward->setEnabled(enable);
        action_moveBackward->setEnabled(enable);
    }
}


void IndexWindow::view_pictures_clicked(QIconViewItem* it)
{
    if (it)
        statusBar()->message(((PictureItem*)it)->getPicture()->fileName());
    else
        statusBar()->clear();
}


void IndexWindow::view_pictures_rightButtonClicked(QIconViewItem* it,
                                                   const QPoint& p)
{
    if (it) {
        PictureContextMenu* m = new PictureContextMenu(0,(PictureItem*) it);
        m->exec(p);
        delete m;
    }
}


void IndexWindow::action_selectAll_activated()
{
    view_pictures->selectAll(true);
}


void IndexWindow::action_selectNone_activated()
{
    view_pictures->clearSelection();
}

void IndexWindow::action_invertSelection_activated()
{
    view_pictures->invertSelection();
}


void IndexWindow::action_deletePictures_activated()
{
    QPtrList<PictureItem> *list = getSelectedPictures();
    
    /* Check delete */
    if (list->count() == 0) return;
    bool removeFile = false;
    switch(QMessageBox::information(this,tr("Deleting Pictures"),
             (list->count() > 1 ? 
                QString(tr("About to delete the selected picture.\n"))
                                                    .arg(list->count())
                : QString(tr("About to delete the selected pictures.\n")))
             + tr("Do you want to delete them from your disk too ?"),
             QMessageBox::Yes, QMessageBox::No | QMessageBox::Default, 
             QMessageBox::Cancel | QMessageBox::Escape)) {
        case QMessageBox::Yes : removeFile = true; break;
        case QMessageBox::No : removeFile = false; break;
        case QMessageBox::Cancel : return;
    }
    
    /* Remove them */
    QPtrListIterator<PictureItem> it(*list);
    for ( ; it.current(); ++it ) {
        (*it)->remove(removeFile);
        delete (*it);
    }

    /* Sort the picture view */
    view_pictures->sort();

    /* Save everything */
    action_saveIndex_activated();
    
    delete list;
}


void IndexWindow::action_changeSettings_activated()
{
     SettingsDialog d;
     d.setBook(book());
     d.exec();
}

void IndexWindow::action_about_activated()
{
    QMessageBox::about( this, "Spit " SPIT_VERSION,
        QString("Spit v" SPIT_VERSION "\n\n"
        "Spit is a simple picture indexing tool.\n\n"
        "(C) 2004 Remko Tronon <spike@ulyssis.org>\n" 
        SPIT_URL "\n\n") + 
        tr("Released under the GPL License."));
}

void IndexWindow::action_aboutQt_activated()
{
    QMessageBox::aboutQt(this);
}

void IndexWindow::action_moveBackward_activated()
{
    QPtrList<PictureItem> *list = getSelectedPictures();
    if (list->getFirst() != view_pictures->firstItem()) {
        QPtrListIterator<PictureItem> it(*list);
        for ( ; it.current(); ++it ) {
            (*it)->moveBackward();
        }
    }
    view_pictures->sort();
    view_pictures->ensureItemVisible(list->getFirst());
    delete list;
}

void IndexWindow::action_moveForward_activated()
{
    QPtrList<PictureItem> *list = getSelectedPictures();
    if (list->getLast() != view_pictures->lastItem()) {
        QPtrListIterator<PictureItem> it(*list);
        for ( it.toLast(); it.current(); --it ) {
            (*it)->moveForward();
        }
    }       
    view_pictures->sort();
    view_pictures->ensureItemVisible(list->getLast());
    delete list;
}


/* Enables all widgets */
void IndexWindow::enableAll()
{
    action_saveIndex->setEnabled(true);
    action_saveIndexAs->setEnabled(true);
    action_generateHTML->setEnabled(true);
    action_previewHTML->setEnabled(true);
    action_importPictures->setEnabled(true);
    action_changeSettings->setEnabled(true);
    action_closeIndex->setEnabled(true);
    text_title->setEnabled(true);
    text_date->setEnabled(true);
    view_pictures->setEnabled(true);
    action_selectAll->setEnabled(true);
    action_selectNone->setEnabled(true);
}


/* Disables all widgets */
void IndexWindow::disableAll() 
{
    action_saveIndex->setEnabled(false);
    action_saveIndexAs->setEnabled(false);
    action_generateHTML->setEnabled(false);
    action_previewHTML->setEnabled(true);
    action_importPictures->setEnabled(false);
    action_changeSettings->setEnabled(false);
    action_closeIndex->setEnabled(false);
    text_title->setEnabled(false);
    text_title->clear();
    text_date->setEnabled(false);
    text_date->clear();
    view_pictures->clear();
    view_pictures_selectionChanged();
    view_pictures->setEnabled(false);
    action_selectAll->setEnabled(false);
    action_selectNone->setEnabled(false);
}


/* Updates pictures in listview */
void IndexWindow::updatePictures() 
{
    view_pictures->clear();
    const QPtrList<Picture>* pics = book()->getPictures();
    QProgressDialog progress(tr("Initializing pictures ..."),0,
        pics->count(),0,"progress",true);
    progress.setMinimumDuration(0);
    
    QPtrListIterator<Picture> it(*pics);
    int i=0;
    for ( ; it.current(); ++it ) {
        progress.setProgress(i++);
        qApp->processEvents();
        new PictureItem(view_pictures,(*it));
    }
    progress.setProgress(pics->count());
}
